
# q-expansion of newform 3528.2.a.f, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3528
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-2], [0], [6], [-3], [-4], [-5], [4], [4], [7], [-9], [2], [-1], [-2], [-8], [0], [10], [-15], [6], [-11], [1], [-6], [8], [-14], [6], [-9], [12], [-11], [-6], [-1], [-14], [-20], [-9], [-4], [-8], [18], [4], [-18], [-20], [-26], [-7], [-10], [3], [12], [-16], [-4], [24], [-14], [-7], [-26], [-2], [-2], [-4], [18], [12], [18], [8], [1], [0], [-1], [0], [-11], [18], [-1], [0], [5], [29], [-12], [-22], [-6], [12], [-7], [-13], [-15], [0], [26], [-5], [0], [-3], [-26], [-35], [18], [31], [24], [16], [38], [13], [12], [17], [-30], [-16], [25], [-36], [-17], [14], [-6], [12], [29], [1], [4], [2], [2], [18], [23], [39], [-16], [30], [-4], [31], [1], [-38], [6], [9], [-40], [20], [-17], [18], [22], [40], [35], [7], [-12], [0], [-7], [-28], [-50], [30], [5], [-11], [-5], [34], [-37], [10], [12], [7], [50], [-32], [-12], [30], [24], [18], [8], [-18], [-43], [-12], [-17], [0], [16], [-14], [2], [-52], [1], [54], [17], [-28], [1], [-58], [-33], [-48], [38], [8], [-27], [56], [18], [0], [-33], [-17], [-27], [8], [-16], [30], [12], [61], [-45], [40], [-7], [-54], [40], [19], [43], [-56], [-35], [-16], [-6], [12], [38], [-19], [33], [66], [-7], [6], [-16], [24], [8], [24], [-53], [14], [14], [56], [-14], [-15], [38], [-38], [-42], [30], [-40], [6], [-44], [28], [-33], [-24], [-20], [24], [64], [-57], [24], [-44], [-48], [26], [57], [43], [58], [-48], [50], [50], [-48], [-14], [1], [30], [25], [-52], [1], [-70], [-53], [-66], [5], [-6], [-4], [30], [-14], [-61], [-37], [-50], [24], [-64], [-11], [70], [-48], [-8], [17], [-44], [74], [-51], [-40], [48], [-43], [53], [42], [-18], [-67], [-48], [9], [-65], [-54], [35], [62], [2], [36], [18], [-17], [-20], [-19], [9], [-30], [44], [-26], [-29], [35], [-60], [-24], [-55], [66], [27], [20], [-18], [9], [-10], [75], [-6], [36], [-4], [46], [-4], [-1], [-22], [39], [-80], [-6], [-19], [86], [-18], [36], [36], [63], [29], [-42], [-11], [8], [71], [40], [10], [62], [-8], [88], [73], [-34], [-4], [1], [-18], [-25], [47], [18], [-23], [58], [-7], [-11], [-89], [-12], [-66], [45], [54], [-17], [50], [-55], [-26], [-86], [20], [53], [-29], [75], [-48], [12], [-13], [-50], [-18], [61], [28], [38], [-2], [-52], [-14], [-42], [-48], [-45], [90], [-30], [-80], [-84], [-90], [26], [-42], [-66], [-30], [51], [-34], [-96], [39], [-50], [88], [-97], [6], [50], [-47], [21], [-28], [-60], [86], [100], [-23], [68], [28], [40], [-72], [44], [-36], [-31], [38], [-7], [2], [41], [50], [-84], [-27], [-22], [2], [-20], [62], [-11], [-66], [30], [-56], [-72], [-42], [-50], [-1], [62], [68], [39], [-84], [18], [84], [16], [4], [57], [30], [26], [-41], [36], [-52], [-12], [11], [-24], [30], [30], [-30], [14], [-48], [92], [32]]
